#include <iostream>
#include <fstream>
#include <string>
#include <math.h>
#include "EasyBMP.h"

using namespace std;

int main(int argc, char **argv)
{
	if(argc != 2)
	{
		cout << "Usage: align_text <BMP-file>" << endl;
		return 1;
	}
	string filename = argv[1];
	BMP img;
	img.ReadFromFile(filename.c_str());
	int width = img.TellWidth();
	int height = img.TellHeight();
	
	int charsize = 18;
	int alignmentpoints[][2] = {
			{7,14},
			{144,9},
			{331,14},
			{500,24}
		};
	int alignmentpoints_count = 4;
	double *centers = new double[width];
	
	fstream csv;
	csv.open("centers.csv", ios::trunc | ios::out);
	
	for(int c=0;c < width; c++)
	{
		double slope;
		if(c <= alignmentpoints[0][0])
		{
			slope = ((double)(alignmentpoints[1][1]-alignmentpoints[0][1])/(double)(alignmentpoints[1][0]-alignmentpoints[0][0]));
			centers[c] = (double)alignmentpoints[0][1]+(double)(c-alignmentpoints[0][0])*slope;
			csv << centers[c] << ";";
		}
		else if(c > alignmentpoints[0][0] && c <= alignmentpoints[alignmentpoints_count-1][0])
		{
			int point_index = 0;
			while(point_index-1 < alignmentpoints_count && alignmentpoints[point_index][0] <= c )
			{
				point_index++;
			}
			if(point_index < 1)
			{
				point_index = 1;
			}
			slope = ((double)(alignmentpoints[point_index][1]-alignmentpoints[point_index-1][1])/(double)(alignmentpoints[point_index][0]-alignmentpoints[point_index-1][0]));
			centers[c] = (double)alignmentpoints[point_index-1][1]+(double)(c-alignmentpoints[point_index-1][0])*slope;
			csv << centers[c] << ";";
		}
		else
		{
			slope = ((double)(alignmentpoints[3][1]-alignmentpoints[2][1])/(double)(alignmentpoints[3][0]-alignmentpoints[2][0]));
			centers[c] = (double)alignmentpoints[3][1]+(double)(c-alignmentpoints[3][0])*slope;
			csv << centers[c] << ";";
		}
	}
	csv.close();
		
	BMP out;
	out.SetSize(width,charsize);
	
	for(int x = 0; x < width; x++)
	{
		int pixel_output = 0;
		for(int y = 0; y < height; y++)
		{
			if(abs(centers[x]-(double)y) < (double)charsize/2.0)
			{
				out(x,pixel_output)->Red = img(x,y)->Red;
				out(x,pixel_output)->Green = img(x,y)->Green;
				out(x,pixel_output)->Blue = img(x,y)->Blue;
				pixel_output++;
			}
			if(pixel_output == charsize)
			{
				break;
			}
		}
	}
	out.WriteToFile("aligned.bmp");
	cout << "Aligned BMP created!" << endl;
	
	return 0;
}