#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <math.h>
#include "../EasyBMP.h"

using namespace std;

int main(int argc, char **argv)
{
	if(argc != 2)
	{
		cout << "Usage: detect_character <BMP-file>" << endl;
		return 1;
	}
	string filename = argv[1];
	BMP img;
	img.ReadFromFile(filename.c_str());
	int width = img.TellWidth();
	int height = img.TellHeight();
	double avg_img_color;
	
	int img_color_sum = 0;
	for(int y = 0; y < height; y++)
	{
		for(int x = 0; x < width; x++)
		{
			img_color_sum += img(x,y)->Red;
		}
	}
	avg_img_color = (double)img_color_sum/(double)(width*height);
	
	double ratio = (double)height/(double)width;
	int charclass = 0;
	
	if(ratio < 1.0)
	{
		cout << "One of: m,w or more than one character!" << endl;
		charclass = 0;
	}
	if(ratio >= 1.0 && ratio <= 1.5)
	{
		cout << "One of: a,c,e,g,n,o,r,s,u,v,x,z" << endl;
		charclass = 1;
	}
	if(ratio > 1.5 && ratio < 1.9)
	{
		cout << "One of: b,d,f,g,h,k,p,q,y and almost all upper case" << endl;
		charclass = 2;
	}
	if(ratio >= 1.9)
	{
		cout << "One of: i,j,l,t" << endl;
		charclass = 3;
	}
	
	string class1[4] = {"mw","acegnorsuvxzOQXZMW","bdfghkpqyABCDEFGHJKLNPRSTUVY","ijltI"};
	
	char alphabet[26] = {'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z'};
	double max_probability = 0.;
	char most_probabable_character;
	for(int i=0;i < 26; i++)
	{
		stringstream ss;
		string ideal_image_path;
		ss << "ideal_characters/" << alphabet[i] << ".bmp";
		ss >> ideal_image_path;
		
		BMP ideal_image;
		ideal_image.ReadFromFile(ideal_image_path.c_str());
		int ideal_width = ideal_image.TellWidth();
		int ideal_height = ideal_image.TellHeight();
		double avg_ideal_color;
		
		double width_ratio = (double)ideal_width/(double)width;
		double height_ratio = (double)ideal_height/(double)height;
		
		int ideal_color_sum = 0;
		for(int y = 0; y < ideal_height; y++)
		{
			for(int x = 0; x < ideal_width; x++)
			{
				ideal_color_sum += ideal_image(x,y)->Red;
			}
		}
		avg_ideal_color = (double)ideal_color_sum/(double)(ideal_width*ideal_height);
		
		BMP out;
		out.SetSize(ideal_width,ideal_height);
		
		double probability = 0.;
		
		for(int y_ideal = 0; y_ideal < ideal_height; y_ideal++)
		{
			double y_img = (double)y_ideal/height_ratio;
			double bottom_weight = y_img-floor(y_img);
			double top_weight = 1.0 - bottom_weight;
			
			for(int x_ideal = 0; x_ideal < ideal_width; x_ideal++)
			{
				double x_img = (double)x_ideal/width_ratio;
				double right_weight = x_img-floor(x_img);
				double left_weight = 1.0 - right_weight;
				
				int img_color;
				if(floor(x_img) >= 0 && floor(y_img) >= 0 && floor(x_img)+1 < width && floor(y_img) < height)
				{
					img_color = (int)(left_weight*(double)(img(floor(x_img),floor(y_img))->Red) + right_weight*(double)(img(floor(x_img)+1,floor(y_img))->Red));
					if(i == 0)
						out(x_ideal,y_ideal)->Red = img_color;
				}
				else
				{
					img_color = (int)( left_weight*(double)(img(floor(x_img),floor(y_img))->Red) + right_weight*255. );
					if(i == 0)
						out(x_ideal,y_ideal)->Red = img_color;
				}
				int ideal_color = ideal_image(x_ideal,y_ideal)->Red;
				probability += ((double)img_color-avg_img_color)*((double)ideal_color-avg_ideal_color);
			}
		}
		probability *= 1./(double)(ideal_width*ideal_height);
		cout << alphabet[i] << ": " << probability << endl;
		if(probability > max_probability)
		{
			max_probability = probability;
			most_probabable_character = alphabet[i];
		}
		if(i == 0)
			out.WriteToFile("out.bmp");
	}
	if(max_probability > 0.0)
		cout << "The character is most likely a '" << most_probabable_character << "'!" << endl;
	else
		cout << "The character is most likely a 'l'!" << endl;
	
	return 0;
}